/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2001.  All Rights Reserved.
 *
 * Protege-2000 was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu
 *
 * Contributor(s):
 */

package edu.stanford.smi.protege;

import java.awt.*;
import java.awt.event.*;
import java.net.*;

import javax.swing.*;

import edu.stanford.smi.protege.resource.*;
import edu.stanford.smi.protege.ui.*;
import edu.stanford.smi.protege.util.*;

/**
 * Main class for the Protege application (as opposed to an applet).
 *
 * @author Ray Fergerson
 * @author Jennifer Vendetti
 */
public class Application {
    private final static char OPTION_CHAR = '-';
    private final static String INVERT_PREFIX = "no";
    private final static String TRACE_PREFIX = "trace";
    private final static String WARNINGS_PREFIX = "warn";
    private final static String STACK_PREFIX = "stack";

    private static JFrame _mainFrame;
    private static SplashScreen _splashScreen;
    private static WelcomeDialog _welcome;

    private static void initialize() {
        try {
            // System.setProperty("winlaf.forceTahoma", "true");
            SystemUtilities.initialize();
            // net.java.plaf.LookAndFeelPatchManager.initialize();
        } catch (Exception e) {
            Log.exception(e, Application.class, "initialize");
        }
    }

    public static Component getMainWindow() {
        Component result;
        if (_splashScreen != null) {
            result = _splashScreen;
        } else {
            result = _mainFrame;
        }
        return result;
    }

    private static URI getProjectURI(String[] args) {
        URI uri = null;
        for (int i = 0; i < args.length; ++i) {
            String arg = args[i];
            if (arg.charAt(0) != OPTION_CHAR) {
                String projectString = arg;
                if (!projectString.endsWith(".pprj")) {
                    projectString += ".pprj";
                }
                uri = URIUtilities.createURI(projectString);
                break;
            }
        }

        return uri;
    }

    private static void init(String[] args) throws Exception {
        parseOptions(args);

        // Construct the application's main frame.
        _mainFrame = ComponentFactory.createMainFrame();
        _mainFrame.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent event) {
                ProjectManager.getProjectManager().exitApplicationRequest();
            }
        });

        restoreMainframeRectangle();
        ProjectManager.getProjectManager().setRootPane(_mainFrame.getRootPane());

        // Find out if a filename was passed in on the command-line.

        URI projectURI = getProjectURI(args);

        if (projectURI != null) {
            // Load the project and show the main frame.
            ProjectManager.getProjectManager().loadProject(projectURI);
            showMainFrame();
        } else {
            // Check to see if the user wants to see the welcome dialog.
            boolean b = ApplicationProperties.getWelcomeDialogShow();
            if (b == true) {
                // Load the main frame and show the welcome dialog.
                _welcome = new WelcomeDialog(_mainFrame, Text.getProgramName(), true);
                showMainFrame();
                _welcome.setLocationRelativeTo(_mainFrame);
                _welcome.setVisible(true);
            } else {
                // Just show Protege w/out the welcome dialog.
                showMainFrame();
            }
        }
    }

    private static void restoreMainframeRectangle() {
        ApplicationProperties.restoreMainFrameProperties(_mainFrame);

        GraphicsEnvironment ge = GraphicsEnvironment.getLocalGraphicsEnvironment();
        GraphicsDevice gd = ge.getDefaultScreenDevice();
        GraphicsConfiguration gc = gd.getDefaultConfiguration();
        Rectangle r = gc.getBounds();

        // make sure that the size is smaller than the current screen size
        Dimension frameSize = _mainFrame.getSize();
        frameSize.width = Math.min(frameSize.width, r.width);
        frameSize.height = Math.min(frameSize.height, r.height);
        _mainFrame.setSize(frameSize);

        // make sure that the position appears on the screen -- some "virtual" windowing systems mess this up.
        Point location = _mainFrame.getLocation();
        location.x = Math.max(r.x, location.x);
        location.y = Math.max(r.y, location.y);
        _mainFrame.setLocation(location);
    }

    public static void main(String[] args) {
        initialize();
        // System.setSecurityManager(new RMISecurityManager());
        // Log.enter(Application.class, "main", args);
        _splashScreen = new SplashScreen();
        try {
            init(args);
        } catch (Exception e) {
            Log.exception(e, Application.class, "main");
        }
        _splashScreen.dispose();
        _splashScreen = null;
    }

    private static void parseOptions(String[] args) {
        for (int i = 0; i < args.length; ++i) {
            String arg = args[i];
            if (arg.length() > 1 && arg.charAt(0) == OPTION_CHAR) {
                boolean invert = false;
                arg = arg.substring(1);
                if (arg.startsWith(INVERT_PREFIX)) {
                    invert = true;
                    arg = arg.substring(INVERT_PREFIX.length());
                }
                if (arg.startsWith(TRACE_PREFIX)) {
                    boolean trace = true;
                    if (invert) {
                        trace = !trace;
                    }
                    Log.setDisplayTrace(trace);
                } else if (arg.startsWith(WARNINGS_PREFIX)) {
                    boolean warnings = true;
                    if (invert) {
                        warnings = !warnings;
                    }
                    Log.setDisplayWarnings(warnings);
                } else if (arg.startsWith(STACK_PREFIX)) {
                    boolean showStack = true;
                    if (invert) {
                        showStack = !showStack;
                    }
                    Log.setShowStack(showStack);
                } else {
                    Log.warning("invalid option: " + arg, Application.class, "parseOptions");
                }
            } else {
                break;
            }
        }
    }

    public static void repaint() {
        if (_mainFrame != null) {
            _mainFrame.repaint();
        }
    }

    private static void showMainFrame() {
        _splashScreen.setVisible(false);
        _mainFrame.setVisible(true);
        ProjectManager.getProjectManager().bringErrorFrameToFront();
    }

    public static void shutdown() {
        _mainFrame.dispatchEvent(new WindowEvent(_mainFrame, WindowEvent.WINDOW_CLOSING));
    }

}
