/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2001.  All Rights Reserved.
 *
 * Protege-2000 was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu
 *
 * Contributor(s):
 */

package edu.stanford.smi.protege.model.framestore;

import java.lang.reflect.*;
import java.util.*;

import edu.stanford.smi.protege.model.*;
import edu.stanford.smi.protege.util.*;

public class DefaultFrameFactory implements FrameFactory {
    private static final Class[] CONSTRUCTOR_PARAMETERS = { KnowledgeBase.class, FrameID.class };
    private KnowledgeBase _kb;
    private Collection _packages = new ArrayList();

    public DefaultFrameFactory(KnowledgeBase kb) {
        _kb = kb;
    }

    public KnowledgeBase getKnowledgeBase() {
        return _kb;
    }

    public void addJavaPackage(String packageName) {
        _packages.add(packageName);
    }

    public void removeJavaPackage(String packageName) {
        _packages.remove(packageName);
    }

    public Cls createCls(FrameID id, Collection directTypes) {
        Cls cls;
        if (_packages.isEmpty()) {
            cls = new DefaultCls(_kb, id);
        } else {
            cls = (Cls) getImplementationInstance(id, directTypes, DefaultCls.class);
        }
        configureCls(cls);
        return cls;
    }

    public Slot createSlot(FrameID id, Collection directTypes) {
        Slot slot;
        if (_packages.isEmpty()) {
            slot = new DefaultSlot(_kb, id);
        } else {
            slot = (Slot) getImplementationInstance(id, directTypes, DefaultSlot.class);
        }
        configureSlot(slot);
        return slot;
    }
    public Facet createFacet(FrameID id, Collection directTypes) {
        Facet facet;
        if (_packages.isEmpty()) {
            facet = new DefaultFacet(_kb, id);
        } else {
            facet = (Facet) getImplementationInstance(id, directTypes, DefaultFacet.class);
        }
        configureFacet(facet);
        return facet;
    }

    public SimpleInstance createSimpleInstance(FrameID id, Collection directTypes) {
        SimpleInstance instance;
        if (_packages.isEmpty()) {
            instance = new DefaultSimpleInstance(_kb, id);
        } else {
            instance = (SimpleInstance) getImplementationInstance(id, directTypes, DefaultSimpleInstance.class);
        }
        configureSimpleInstance(instance);
        return instance;
    }

    protected void configureFacet(Facet facet) {
        if (facet.isSystem()) {
            Facet cachedSystemFacet = (Facet) getCachedSystemFrame(facet.getFrameID());
            if (cachedSystemFacet != null) {
                facet.setConstraint(cachedSystemFacet.getConstraint());
            }
        }
    }

    protected void configureSimpleInstance(SimpleInstance simpleInstance) {
        // do nothing (for now)
    }

    protected void configureCls(Cls cls) {
        // do nothing (for now)
    }

    protected void configureSlot(Slot slot) {
        // do nothing (for now)
    }

    private Frame getCachedSystemFrame(FrameID id) {
        return _kb.getSystemFrames().getFrame(id);
    }

    private Instance getImplementationInstance(FrameID id, Collection directTypes, Class type) {
        Instance instance = null;
        Class javaImplementationClass = getJavaImplementationClass(directTypes);
        if (javaImplementationClass != null) {
            if (type.isAssignableFrom(javaImplementationClass)) {
                instance = createInstance(id, javaImplementationClass);
            } else {
                String text = "Java implementation class of wrong type: " + javaImplementationClass;
                Log.error(text, this, "getJavaImplementationInstance", id, directTypes, type);
            }
        }
        if (instance == null) {
            instance = createInstance(id, type);
        }
        return instance;
    }

    public boolean isCorrectJavaImplementationClass(FrameID id, Collection types, Class clas) {
        boolean isCorrect = true;
        if (!_packages.isEmpty()) {
            Class javaImplementationClass = getJavaImplementationClass(types);
            if (javaImplementationClass != null) {
                isCorrect = javaImplementationClass.equals(clas);
            }
        }
        return isCorrect;
    }

    private Instance createInstance(FrameID id, Class type) {
        Instance instance = null;
        try {
            Constructor constructor = type.getConstructor(CONSTRUCTOR_PARAMETERS);
            instance = (Instance) constructor.newInstance(new Object[] { _kb, id });
        } catch (Exception e) {
            Log.exception(e, this, "createInstance", id, type);
        }
        return instance;
    }

    private Class getJavaImplementationClass(Collection types) {
        Class implementationClass = null;
        Iterator i = _packages.iterator();
        while (i.hasNext() && implementationClass == null) {
            String packageName = (String) i.next();
            implementationClass = getJavaImplementationClass(packageName, types);
        }
        return implementationClass;
    }

    private Class getJavaImplementationClass(String packageName, Collection types) {
        Class implementationClass = null;
        Iterator i = types.iterator();
        while (i.hasNext() && implementationClass == null) {
            Cls type = (Cls) i.next();
            implementationClass = getJavaImplementationClass(packageName, type);
        }
        return implementationClass;
    }

    private Class getJavaImplementationClass(String packageName, Cls type) {
        String typeName = getJavaClassName(type);
        String className = packageName + "." + typeName;
        return SystemUtilities.forName(className, true);
    }

    protected String getJavaClassName(Cls type) {
        StringBuffer className = new StringBuffer();
        String typeName = type.getName();
        for (int i = 0; i < typeName.length(); ++i) {
            char c = typeName.charAt(i);
            if (isValidCharacter(c, className.length())) {
                className.append(c);
            }
        }
        return className.toString();
    }

    protected boolean isValidCharacter(char c, int i) {
        return (i == 0) ? Character.isJavaIdentifierStart(c) : Character.isJavaIdentifierPart(c);
    }
}