/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2001.  All Rights Reserved.
 *
 * Protege-2000 was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu
 *
 * Contributor(s):
 */

package edu.stanford.smi.protege.model.framestore;

import java.util.*;

import edu.stanford.smi.protege.model.*;
import edu.stanford.smi.protege.util.*;

public abstract class FrameStore_Test extends SimpleTestCase {
    private DefaultKnowledgeBase _kb;
    private FrameStore _testFrameStore;
    private FrameStore _modifiableFrameStore;

    public FrameStore_Test() {
    }

    protected void setUp() {
        _kb = new DefaultKnowledgeBase();
        _testFrameStore = createFrameStore(_kb);
        _modifiableFrameStore = getModifiableFrameStore(_kb);
        _kb.setTerminalFrameStore(_testFrameStore);
    }

    protected void tearDown() {
        _kb.close();
        _kb = null;
        _testFrameStore = null;
        _modifiableFrameStore = null;
    }

    protected FrameStore getTestFrameStore() {
        return _testFrameStore;
    }

    protected abstract FrameStore createFrameStore(DefaultKnowledgeBase kb);

    protected FrameStore getModifiableFrameStore(DefaultKnowledgeBase kb) {
        return _testFrameStore;
    }

    private static String createFrameName() {
        return "Frame_" + Math.random();
    }

    protected Cls createCls() {
        Cls parent = (Cls) getFrame(Model.Cls.THING);
        assertNotNull("thing", parent);
        return createCls(parent);
    }

    protected Slot createSlot(Slot slot) {
        return createSlot(null, makeList(slot));
    }

    protected Slot createSlotOnCls(Cls cls) {
        Slot slot = createSlot();
        addTemplateSlot(cls, slot);
        return slot;
    }

    protected Slot createSlotOnCls(Cls cls, ValueType type, boolean allowsMultipleValues) {
        Slot slot = createSlotOnCls(cls);
        setTypeAndCardinality(slot, type, allowsMultipleValues);
        return slot;
    }

    private void setTypeAndCardinality(Slot slot, ValueType type, boolean allowsMultipleValues) {
        Slot valueTypeSlot = (Slot) _testFrameStore.getFrame(Model.Slot.VALUE_TYPE);
        if (equals(type, ValueType.INSTANCE)) {
            Cls thing = (Cls) _testFrameStore.getFrame(Model.Cls.THING);
            _modifiableFrameStore.setDirectOwnSlotValues(slot, valueTypeSlot, makeList(type.toString(), thing));
        } else {
            _modifiableFrameStore.setDirectOwnSlotValues(slot, valueTypeSlot, makeList(type.toString()));
        }
        Slot maxCardinalitySlot = (Slot) _testFrameStore.getFrame(Model.Slot.MAXIMUM_CARDINALITY);
        Collection values = allowsMultipleValues ? Collections.EMPTY_SET : Collections.singleton(new Integer(1));
        _modifiableFrameStore.setDirectOwnSlotValues(slot, maxCardinalitySlot, values);
    }

    protected Slot createSlotOnCls(Cls cls, Cls directType) {
        Slot slot = createSlot(null, Collections.EMPTY_SET, directType);
        addTemplateSlot(cls, slot);
        return slot;
    }

    protected void addTemplateSlot(Cls cls, Slot slot) {
        _modifiableFrameStore.addDirectTemplateSlot(cls, slot);
    }

    protected SimpleInstance createSimpleInstance(Cls type) {
        return createSimpleInstance(null, type);
    }

    private static FrameID getID() {
        return null;
    }

    protected SimpleInstance createSimpleInstance(String name, Cls type) {
        Collection types = makeList(type);
        return _modifiableFrameStore.createSimpleInstance(getID(), name, types, true);
    }

    protected Cls createCls(Cls parent) {
        Cls directType = (Cls) _testFrameStore.getFrame(Model.Cls.STANDARD_CLASS);
        assertNotNull("directType", directType);
        Collection types = makeList(directType);
        Collection parents = makeList(parent);
        return _modifiableFrameStore.createCls(getID(), null, types, parents, true);
    }

    protected Slot createSlot() {
        return createSlot((String) null);
    }

    protected Slot createSlot(String name) {
        return createSlot(name, Collections.EMPTY_LIST);
    }

    protected Slot createSlot(String name, Collection superslots) {
        Cls type = (Cls) _testFrameStore.getFrame(Model.Cls.STANDARD_SLOT);
        return createSlot(name, superslots, type);
    }
    protected Slot createSlot(String name, Collection superslots, Cls type) {
        Collection types = makeList(type);
        return _modifiableFrameStore.createSlot(getID(), name, types, superslots, true);
    }

    protected Facet createFacet() {
        return createFacet(ValueType.STRING);
    }
    protected Facet createFacet(ValueType valueType) {
        Cls type = (Cls) _testFrameStore.getFrame(Model.Cls.STANDARD_FACET);
        return createFacet(valueType, type);
    }

    protected Facet createFacet(ValueType valueType, Cls directType) {
        Collection types = makeList(directType);
        Facet facet = _modifiableFrameStore.createFacet(getID(), null, types, true);
        Cls defaultMetaSlot = (Cls) _testFrameStore.getFrame(Model.Cls.STANDARD_SLOT);
        Cls metaCls = createCls(defaultMetaSlot);
        Slot associatedSlot = createSlotOnCls(metaCls);
        setTypeAndCardinality(associatedSlot, valueType, true);
        Slot associatedFacetSlot = (Slot) _testFrameStore.getFrame(Model.Slot.ASSOCIATED_FACET);
        _modifiableFrameStore.setDirectOwnSlotValues(associatedSlot, associatedFacetSlot, makeList(facet));
        return facet;
    }

    protected Frame getFrame(String s) {
        return _testFrameStore.getFrame(s);
    }

    protected String getName(Frame frame) {
        return _testFrameStore.getFrameName(frame);
    }

    protected void setOwnSlotValue(Frame frame, Slot slot, Object value) {
        Collection values = CollectionUtilities.createCollection(value);
        _modifiableFrameStore.setDirectOwnSlotValues(frame, slot, values);
    }

    // ------------------------ test cases -------------------------------------------
    public void testCreateCls() {
        String name = createFrameName();
        Cls directType = (Cls) _testFrameStore.getFrame(Model.Cls.STANDARD_CLASS);
        Assert.assertNotNull("directType", directType);
        Cls parent = (Cls) _testFrameStore.getFrame(Model.Cls.THING);
        Collection parents = makeList(parent);
        Collection types = makeList(directType);
        Cls cls = _testFrameStore.createCls(getID(), name, types, parents, true);
        assertEquals("name", name, getName(cls));
        assertEquals("direct type", directType, cls.getDirectType());
        assertEqualsList("parents", parents, cls.getDirectSuperclasses());
        Cls fsCls = (Cls) _testFrameStore.getFrame(name);
        assertEquals("pointer", cls, fsCls);
    }

    public void testGetFrame() {
        Frame frame = createCls();
        String name = _testFrameStore.getFrameName(frame);
        assertNotNull("name", name);
        assertEquals(frame, _testFrameStore.getFrame(name));
    }

    public void testCreateSlot() {
        Cls standardSlot = (Cls) getFrame(Model.Cls.STANDARD_SLOT);
        Collection standardSlots = makeList(standardSlot);
        Slot slota = _testFrameStore.createSlot(getID(), null, standardSlots, Collections.EMPTY_LIST, true);
        assertNotNull(slota);
        assertEquals("slota", slota, getFrame(getName(slota)));
        List types = _testFrameStore.getDirectTypes(slota);
        assertEquals("types size", 1, types.size());
        assertEquals("standard slot", standardSlot, types.iterator().next());
        Slot slotb = _testFrameStore.createSlot(getID(), null, standardSlots, Collections.EMPTY_LIST, true);
        List superslots = makeList(slota, slotb);
        String slotName = createFrameName();
        Slot slotc = _testFrameStore.createSlot(getID(), slotName, standardSlots, superslots, true);
        assertNotNull(slotc);
        assertEquals("slotc", slotc, getFrame(getName(slotc)));
        assertEqualsSet("superslots", superslots, _testFrameStore.getDirectSuperslots(slotc));
    }
    public void testCreateSimpleInstance() {
        Cls cls = createCls();
        Collection types = makeList(cls);
        SimpleInstance simpleInstance = _testFrameStore.createSimpleInstance(getID(), null, types, true);
        assertNotNull(simpleInstance);
        assertEquals(simpleInstance, getFrame(getName(simpleInstance)));
    }
    public void testCreateFacet() {
        Cls cls = (Cls) getFrame(Model.Cls.STANDARD_FACET);
        Collection types = makeList(cls);
        Facet facet = _testFrameStore.createFacet(getID(), null, types, true);
        assertNotNull(facet);
        assertEquals(facet, getFrame(getName(facet)));
    }

    public void testDeleteCls() {
        Cls cls = createCls();
        assertNotNull(cls);
        String name = getName(cls);
        assertNotNull(name);
        _testFrameStore.deleteCls(cls);
        assertNull("no class after delete", getFrame(name));
    }
    public void testDeleteSlot() {
        String name = "deleteSlot";
        Collection types = makeList(getFrame(Model.Cls.STANDARD_SLOT));
        Slot slot = _testFrameStore.createSlot(getID(), name, types, Collections.EMPTY_LIST, true);
        assertNotNull(slot);
        _testFrameStore.deleteSlot(slot);
        assertNull("no slot after delete", getFrame(name));
    }
    public void testDeleteFacet() {
        String name = "deleteFacet";
        Collection types = makeList(getFrame(Model.Cls.STANDARD_FACET));
        Facet facet = _testFrameStore.createFacet(getID(), name, types, true);
        assertNotNull(facet);
        _testFrameStore.deleteFacet(facet);
        assertNull("no facet after delete", getFrame(name));
    }
    public void testDeleteSimpleInstance() {
        Collection types = makeList(createCls());
        String name = "deleteSimpleInstance";
        SimpleInstance simpleInstance = _testFrameStore.createSimpleInstance(getID(), name, types, true);
        assertNotNull(simpleInstance);
        _testFrameStore.deleteSimpleInstance(simpleInstance);
        assertNull("no simpleInstance after delete", getFrame(name));
    }
    public void testGetClses() {
        Collection clsesStart = new ArrayList(_testFrameStore.getClses());
        Cls thing = (Cls) getFrame(Model.Cls.THING);
        assertNotNull("thing not null", thing);
        assertTrue("thing", clsesStart.contains(thing));
        Cls cls = createCls();
        Collection clsesEnd = _testFrameStore.getClses();
        assertEquals("size", clsesStart.size() + 1, clsesEnd.size());
        assertTrue("new class", clsesEnd.contains(cls));
    }

    public void testGetInstances() {
        Cls cls = createCls();
        Collection instances = _testFrameStore.getInstances(cls);
        assertEquals("none", 0, instances.size());
        Collection types = makeList(cls);
        _modifiableFrameStore.createSimpleInstance(getID(), null, types, true);
        instances = _testFrameStore.getInstances(cls);
        assertEquals("after", 1, instances.size());
    }

    public void testAddDirectTemplateSlot() {
        Cls cls = createCls();
        Slot slot = createSlot();
        _testFrameStore.addDirectTemplateSlot(cls, slot);
        List slots = _testFrameStore.getDirectTemplateSlots(cls);
        assertEquals("size", 1, slots.size());
        assertEquals("value", slot, slots.get(0));
    }

    public void testGetOwnSlots() {
        Cls cls = createCls();
        createSlotOnCls(cls);
        Cls subClass = createCls(cls);
        createSlotOnCls(subClass);
        Collection subclasses = makeList(subClass);
        SimpleInstance instance = _modifiableFrameStore.createSimpleInstance(getID(), null, subclasses, true);
        Set ownSlots = _testFrameStore.getOwnSlots(instance);
        Set templateSlots = _testFrameStore.getTemplateSlots(subClass);
        assertEquals("size", 2, templateSlots.size());
        Set testOwnSlots = new HashSet(templateSlots);
        testOwnSlots.add(getFrame(Model.Slot.NAME));
        testOwnSlots.add(getFrame(Model.Slot.DIRECT_TYPES));
        assertEquals("sets", ownSlots, testOwnSlots);
    }
    public void testGetTemplateSlots() {
        Cls cls = createCls();
        Slot slot = createSlotOnCls(cls);
        Cls subClass = createCls(cls);
        Slot slotb = createSlotOnCls(subClass);
        Cls subSubClass = createCls(subClass);
        Collection slots = _modifiableFrameStore.getTemplateSlots(subSubClass);
        assertEquals("size", 2, slots.size());
        assertTrue("slot", slots.contains(slot));
        assertTrue("slotb", slots.contains(slotb));
    }
    public void testGetDirectOwnSlotValues() {
        Cls cls = createCls();
        Slot slot = createSlotOnCls(cls, ValueType.STRING, true);
        Collection classes = makeList(cls);
        SimpleInstance instance = _modifiableFrameStore.createSimpleInstance(getID(), null, classes, true);
        List values = makeList("foo", "bar");
        List oldValues = _testFrameStore.getDirectOwnSlotValues(instance, slot);
        _modifiableFrameStore.setDirectOwnSlotValues(instance, slot, values);
        assertEquals("oldsize", 0, oldValues.size());
        List newValues = _testFrameStore.getDirectOwnSlotValues(instance, slot);
        assertEquals("newsize", values.size(), newValues.size());
        assertEquals("lists", values, newValues);
    }

    public void testGetReferences() {
        Cls cls = createCls();
        Slot slota = createSlotOnCls(cls, ValueType.INSTANCE, true);
        Slot slotb = createSlotOnCls(cls, ValueType.INSTANCE, true);
        Slot dislot = (Slot) _testFrameStore.getFrame(Model.Slot.DIRECT_INSTANCES);
        Facet facet = createFacet(ValueType.INSTANCE);
        Instance instanceA = createSimpleInstance(cls);
        Instance instanceB = createSimpleInstance(cls);
        Collection instances = makeList(instanceB);
        _modifiableFrameStore.setDirectOwnSlotValues(instanceA, slota, instances);
        _modifiableFrameStore.setDirectTemplateSlotValues(cls, slotb, instances);
        _modifiableFrameStore.setDirectTemplateFacetValues(cls, slota, facet, instances);
        Collection references = _testFrameStore.getReferences(instanceB);
        assertTrue("own slot value", references.contains(new ReferenceImpl(instanceA, slota, null, false)));
        // assertTrue("template slot value", references.contains(new ReferenceImpl(cls, slotb, null, true)));
        assertTrue("type", references.contains(new ReferenceImpl(cls, dislot, null, false)));
        assertTrue("facet value", references.contains(new ReferenceImpl(cls, slota, facet, true)));
        assertEquals("size", 4, references.size());
    }
    public void testGetFrames() {
        Frame thing = _testFrameStore.getFrame(Model.Cls.THING);
        Collection oldFrames = _testFrameStore.getFrames();
        int size = oldFrames.size();
        assertTrue("contains thing", oldFrames.contains(thing));
        Cls cls = createCls();
        createSlot();
        createSimpleInstance(cls);
        Collection frames = _testFrameStore.getFrames();
        assertEquals("size", size + 3, frames.size());
    }

    public void testGetFrameName() {
        String testName = createFrameName();
        assertNull("before create", _testFrameStore.getFrame(testName));
        Cls cls = createCls();
        _modifiableFrameStore.setFrameName(cls, testName);
        assertEquals("frame", cls, _testFrameStore.getFrame(testName));
        assertEquals("name", testName, _testFrameStore.getFrameName(cls));
    }

    public void testSetFrameName() {
        String name = createFrameName();
        Cls cls = createCls();
        String oldName = _testFrameStore.getFrameName(cls);
        assertNull("before", _testFrameStore.getFrame(name));
        _testFrameStore.setFrameName(cls, name);
        assertEquals("new name", cls, _testFrameStore.getFrame(name));
        assertNull("oldName", _testFrameStore.getFrame(oldName));
    }

    public void testGetOwnSlotValues() {
        Cls cls = createCls();
        Slot slot = createSlotOnCls(cls, ValueType.STRING, true);
        List templateValues = makeList("foo", "bar");
        _modifiableFrameStore.setDirectTemplateSlotValues(cls, slot, templateValues);
        Instance instance = createSimpleInstance(cls);
        List ownValues = makeList("foob", "baz");
        _modifiableFrameStore.setDirectOwnSlotValues(instance, slot, ownValues);
        Collection values = _testFrameStore.getOwnSlotValues(instance, slot);
        Set originalValues = new HashSet(templateValues);
        originalValues.addAll(ownValues);
        assertEqualsSet("size", originalValues, values);
    }

    public void testSetDirectOwnSlotValues() {
        Cls cls = createCls();
        Slot slot = createSlotOnCls(cls, ValueType.STRING, true);
        Instance instance = createSimpleInstance(cls);
        Collection values = makeList("foo", "bar");
        Collection oldValues = _testFrameStore.getDirectOwnSlotValues(instance, slot);
        assertEquals("no old values", 0, oldValues.size());
        _testFrameStore.setDirectOwnSlotValues(instance, slot, values);
        Collection newValues = _testFrameStore.getDirectOwnSlotValues(instance, slot);
        assertEqualsList("values", values, newValues);
        _testFrameStore.setDirectOwnSlotValues(instance, slot, Collections.EMPTY_SET);
        Collection emptyValues = _testFrameStore.getDirectOwnSlotValues(instance, slot);
        assertEquals("empty values", 0, emptyValues.size());
    }
    public void testGetDirectTemplateSlots() {
        Cls cls = createCls();
        Collection oldSlots = _testFrameStore.getDirectTemplateSlots(cls);
        assertEquals("old values", 0, oldSlots.size());
        Slot slota = createSlotOnCls(cls);
        Slot slotb = createSlotOnCls(cls);
        Collection slots = _testFrameStore.getDirectTemplateSlots(cls);
        assertEquals("size", 2, slots.size());
        assertTrue("a", slots.contains(slota));
        assertTrue("b", slots.contains(slotb));
    }
    public void testRemoveDirectTemplateSlot() {
        Cls cls = createCls();
        Slot slota = createSlotOnCls(cls);
        Slot slotb = createSlotOnCls(cls);
        Slot slotc = createSlotOnCls(cls);
        assertEquals("size1", 3, _testFrameStore.getDirectTemplateSlots(cls).size());
        _testFrameStore.removeDirectTemplateSlot(cls, slotb);
        Collection newValues = _testFrameStore.getDirectTemplateSlots(cls);
        assertEquals("size2", 2, newValues.size());
        assertTrue("contains a", newValues.contains(slota));
        assertTrue("contains c", newValues.contains(slotc));
        _testFrameStore.removeDirectTemplateSlot(cls, slota);
        _testFrameStore.removeDirectTemplateSlot(cls, slotc);
        assertEquals("size3", 0, _testFrameStore.getDirectTemplateSlots(cls).size());
    }
    public void testMoveDirectTemplateSlot() {
        Cls cls = createCls();
        Slot slota = createSlotOnCls(cls);
        Slot slotb = createSlotOnCls(cls);
        Slot slotc = createSlotOnCls(cls);
        List slots = _testFrameStore.getDirectTemplateSlots(cls);
        assertEqualsList("start", makeList(slota, slotb, slotc), slots);
        _testFrameStore.moveDirectTemplateSlot(cls, slota, 2);
        _testFrameStore.moveDirectTemplateSlot(cls, slotc, 0);
        List movedSlots = _testFrameStore.getDirectTemplateSlots(cls);
        assertEqualsList("end", makeList(slotc, slotb, slota), movedSlots);
    }
    public void testGetTemplateSlotValues() {
        Cls cls = createCls();
        Cls subclass = createCls(cls);
        Slot slot = createSlotOnCls(cls);
        Collection clsValues = makeList("foo");
        Collection subclassValues = makeList("bar");
        Collection allValues = new ArrayList(clsValues);
        allValues.addAll(subclassValues);
        _modifiableFrameStore.setDirectTemplateSlotValues(cls, slot, clsValues);
        Collection clsValues1 = _testFrameStore.getTemplateSlotValues(cls, slot);
        assertEqualsSet("class values", clsValues, clsValues1);
        Collection subclassValues1 = _testFrameStore.getTemplateSlotValues(subclass, slot);
        assertEqualsSet("subclass values1", clsValues, subclassValues1);
        _modifiableFrameStore.setDirectTemplateSlotValues(subclass, slot, subclassValues);
        Collection subclassValues2 = _testFrameStore.getTemplateSlotValues(subclass, slot);
        assertEqualsSet("subclass values2", allValues, subclassValues2);
    }

    public void testGetDirectTemplateSlotValues() {
        Cls cls = createCls();
        Slot slot = createSlotOnCls(cls, ValueType.STRING, true);
        Collection oldValues = makeList("foo", "bar");
        _modifiableFrameStore.setDirectTemplateSlotValues(cls, slot, oldValues);
        List newValues = _testFrameStore.getDirectTemplateSlotValues(cls, slot);
        assertEqualsList("values", oldValues, newValues);
    }

    public void testSetDirectTemplateSlotValues() {
        Cls cls = createCls();
        Slot slot = createSlotOnCls(cls, ValueType.STRING, true);
        Collection oldValues = makeList("foo", "bar");
        _testFrameStore.setDirectTemplateSlotValues(cls, slot, oldValues);
        List newValues = _testFrameStore.getDirectTemplateSlotValues(cls, slot);
        assertEqualsList("values", oldValues, newValues);
    }

    public void testGetTemplateFacetValues() {
        Cls cls = createCls();
        Slot slot = createSlotOnCls(cls, ValueType.STRING, true);
        Facet facet = createFacet();
        Cls subclass = createCls(cls);
        Cls subsubclass = createCls(subclass);
        Collection clsValues = makeList("foo");
        Collection subclassValues = makeList("bar");
        _modifiableFrameStore.setDirectTemplateFacetValues(cls, slot, facet, clsValues);
        _modifiableFrameStore.setDirectTemplateFacetValues(subclass, slot, facet, subclassValues);
        _testFrameStore.getDirectTemplateFacetValues(cls, slot, facet);
        _testFrameStore.getDirectTemplateFacetValues(subclass, slot, facet);
        _testFrameStore.getDirectTemplateFacetValues(subsubclass, slot, facet);
    }
    public void testGetDirectTemplateFacetValues() {
        Cls cls = createCls();
        Slot slot = createSlotOnCls(cls, ValueType.STRING, true);
        Facet facet = createFacet();
        Collection oldValues = makeList("foo", "bar");
        _modifiableFrameStore.setDirectTemplateFacetValues(cls, slot, facet, oldValues);
        Collection newValues = _testFrameStore.getDirectTemplateFacetValues(cls, slot, facet);
        assertEqualsList("values", oldValues, newValues);
    }
    public void testSetDirectTemplateFacetValues() {
        Cls cls = createCls();
        Slot slot = createSlotOnCls(cls, ValueType.STRING, true);
        Facet facet = createFacet();
        Collection oldValues = makeList("foo", "bar");
        _testFrameStore.setDirectTemplateFacetValues(cls, slot, facet, oldValues);
        Collection newValues = _testFrameStore.getDirectTemplateFacetValues(cls, slot, facet);
        assertEqualsList("values", oldValues, newValues);
    }
    public void testGetDirectSuperclasses() {
        Cls cls = createCls();
        Cls cls2 = createCls();
        Cls subclass = createCls(cls);
        _modifiableFrameStore.addDirectSuperclass(subclass, cls2);
        Collection clses = _testFrameStore.getDirectSuperclasses(subclass);
        assertEqualsList("clses", makeList(cls, cls2), clses);
    }
    public void testGetSuperclasses() {
        Cls thing = (Cls) getFrame(Model.Cls.THING);
        Cls cls = createCls(thing);
        Cls cls2 = createCls(thing);
        Cls subclass = createCls(cls);
        Cls subsubclass = createCls(subclass);
        _modifiableFrameStore.addDirectSuperclass(subclass, cls2);
        Collection clses = _testFrameStore.getSuperclasses(subsubclass);
        assertEqualsSet("clses", makeList(thing, cls, cls2, subclass), clses);
    }
    public void testGetDirectSubclasses() {
        Cls cls = createCls();
        Cls subclass1 = createCls(cls);
        Cls subclass2 = createCls(cls);
        Collection clses = _testFrameStore.getDirectSubclasses(cls);
        assertEqualsList("clses", makeList(subclass1, subclass2), clses);
    }
    public void testGetSubclasses() {
        Cls cls = createCls();
        Cls subclass = createCls(cls);
        Cls subsubclass1 = createCls(subclass);
        Cls subsubclass2 = createCls(subclass);
        Collection clses = _testFrameStore.getSubclasses(cls);
        assertEqualsSet("clses", makeList(subclass, subsubclass1, subsubclass2), clses);
    }
    public void testAddDirectSuperclass() {
        Cls cls = createCls();
        Cls cls2 = createCls(cls);
        Cls cls3 = createCls(cls);
        Cls cls4 = createCls(cls2);
        _testFrameStore.addDirectSuperclass(cls4, cls3);
        Collection clses = _testFrameStore.getDirectSuperclasses(cls4);
        assertEqualsSet("clses", makeList(cls2, cls3), clses);
    }

    public void testRemoveDirectSuperclass() {
        Cls cls = createCls();
        Cls cls2 = createCls(cls);
        Cls cls3 = createCls(cls);
        Cls cls4 = createCls(cls);
        Cls cls5 = createCls(cls2);
        _modifiableFrameStore.addDirectSuperclass(cls5, cls3);
        _modifiableFrameStore.addDirectSuperclass(cls5, cls4);
        Collection clses = _testFrameStore.getDirectSuperclasses(cls5);
        assertEqualsSet("clses start", makeList(cls2, cls3, cls4), clses);
        _testFrameStore.removeDirectSuperclass(cls5, cls2);
        clses = _testFrameStore.getDirectSuperclasses(cls5);
        assertEqualsSet("clses end", makeList(cls3, cls4), clses);
    }
    public void testMoveDirectSubclass() {
        Cls cls = createCls();
        Cls clsa = createCls(cls);
        Cls clsb = createCls(cls);
        Cls clsc = createCls(cls);
        List clses = _testFrameStore.getDirectSubclasses(cls);
        assertEqualsList("start", makeList(clsa, clsb, clsc), clses);
        _testFrameStore.moveDirectSubclass(cls, clsa, 1);
        clses = _testFrameStore.getDirectSubclasses(cls);
        assertEqualsList("1", makeList(clsb, clsa, clsc), clses);
        _testFrameStore.moveDirectSubclass(cls, clsa, 2);
        clses = _testFrameStore.getDirectSubclasses(cls);
        assertEqualsList("2", makeList(clsb, clsc, clsa), clses);
        _testFrameStore.moveDirectSubclass(cls, clsa, 0);
        clses = _testFrameStore.getDirectSubclasses(cls);
        assertEqualsList("3", makeList(clsa, clsb, clsc), clses);
    }
    public void testGetDirectSuperslots() {
        Slot slot = createSlot();
        Slot slota = createSlot(slot);
        Slot slotb = createSlot(slot);
        Slot slotc = createSlot(slota);
        _modifiableFrameStore.addDirectSuperslot(slotc, slotb);
        assertEquals("no superslots", 0, _testFrameStore.getDirectSuperslots(slot).size());
        assertEqualsList("1 superslot", makeList(slot), _testFrameStore.getDirectSuperslots(slota));
        assertEqualsList("2 superslots", makeList(slota, slotb), _testFrameStore.getDirectSuperslots(slotc));
    }
    public void testGetSuperslots() {
        Slot slot = createSlot();
        Slot slota = createSlot(slot);
        Slot slotb = createSlot(slot);
        Slot slotc = createSlot(slota);
        _modifiableFrameStore.addDirectSuperslot(slotc, slotb);
        assertEquals("no superslots", 0, _testFrameStore.getSuperslots(slot).size());
        assertEqualsSet("1 superslot", makeList(slot), _testFrameStore.getSuperslots(slota));
        assertEqualsSet("2 superslots", makeList(slot, slota, slotb), _testFrameStore.getSuperslots(slotc));
    }
    public void testGetDirectSubslots() {
        Slot slot = createSlot();
        Slot slota = createSlot(slot);
        Slot slotb = createSlot(slot);
        Slot slotc = createSlot(slota);
        _modifiableFrameStore.addDirectSuperslot(slotc, slotb);
        assertEqualsList("slot", makeList(slota, slotb), _testFrameStore.getDirectSubslots(slot));
        assertEqualsList("slota", makeList(slotc), _testFrameStore.getDirectSubslots(slota));
        assertEqualsList("slotc", makeList(), _testFrameStore.getDirectSubslots(slotc));
    }
    public void testGetSubslots() {
        Slot slot = createSlot();
        Slot slota = createSlot(slot);
        Slot slotb = createSlot(slot);
        Slot slotc = createSlot(slota);
        _modifiableFrameStore.addDirectSuperslot(slotc, slotb);
        assertEqualsSet("slot", makeList(slota, slotb, slotc), _testFrameStore.getSubslots(slot));
        assertEqualsSet("slota", makeList(slotc), _testFrameStore.getDirectSubslots(slota));
        assertEqualsSet("slotc", makeList(), _testFrameStore.getDirectSubslots(slotc));
    }
    public void testAddDirectSuperslot() {
        Slot slot = createSlot();
        Slot slotb = createSlot();
        Slot slotc = createSlot(slot);
        _testFrameStore.addDirectSuperslot(slotb, slot);
        assertEqualsList("b", makeList(slot), _testFrameStore.getDirectSuperslots(slotb));
        _testFrameStore.addDirectSuperslot(slotc, slotb);
        assertEqualsList("c", makeList(slot, slotb), _testFrameStore.getDirectSuperslots(slotc));
    }
    public void testRemoveDirectSuperslot() {
        Slot slota = createSlot();
        Slot slotb = createSlot();
        Slot slotc = createSlot(slota);
        _modifiableFrameStore.addDirectSuperslot(slotc, slotb);
        assertEqualsList("c", makeList(slota, slotb), _testFrameStore.getDirectSuperslots(slotc));
        _testFrameStore.removeDirectSuperslot(slotc, slota);
        assertEqualsList("c1", makeList(slotb), _testFrameStore.getDirectSuperslots(slotc));
        _testFrameStore.removeDirectSuperslot(slotc, slotb);
        assertEqualsList("c0", makeList(), _testFrameStore.getDirectSuperslots(slotc));
    }

    public void testGetDirectTypes() {
        Cls cls = createCls();
        Cls cls2 = createCls();
        Instance instance = createSimpleInstance(cls);
        assertEqualsList("1", makeList(cls), _testFrameStore.getDirectTypes(instance));
        _modifiableFrameStore.addDirectType(instance, cls2);
        assertEqualsList("2", makeList(cls, cls2), _testFrameStore.getDirectTypes(instance));
    }

    public void testGetTypes() {
        Cls thing = (Cls) getFrame(Model.Cls.THING);
        Cls cls = createCls();
        Cls cls2 = createCls();
        Instance instance = createSimpleInstance(cls);
        assertEqualsSet("1", makeList(cls, thing), _testFrameStore.getTypes(instance));
        _modifiableFrameStore.addDirectType(instance, cls2);
        assertEqualsSet("2", makeList(cls, cls2, thing), _testFrameStore.getTypes(instance));
    }

    public void testGetDirectInstances() {
        Cls cls = createCls();
        Instance instancea = createSimpleInstance(null, cls);
        Instance instanceb = createSimpleInstance(null, cls);
        Instance instancec = createSimpleInstance(null, cls);
        assertEqualsList("1", makeList(instancea, instanceb, instancec), _testFrameStore.getDirectInstances(cls));
    }

    public void testAddDirectType() {
        Cls cls = createCls();
        Cls clsb = createCls();
        Instance instance = createSimpleInstance(cls);
        _testFrameStore.addDirectType(instance, clsb);
        assertEqualsList("1", makeList(cls, clsb), _testFrameStore.getDirectTypes(instance));
    }

    public void testRemoveDirectType() {
        Cls cls = createCls();
        Cls clsb = createCls();
        Instance instance = createSimpleInstance(cls);
        _testFrameStore.addDirectType(instance, clsb);
        assertEqualsList("1", makeList(cls, clsb), _testFrameStore.getDirectTypes(instance));
        _testFrameStore.removeDirectType(instance, cls);
        assertEqualsList("2", makeList(clsb), _testFrameStore.getDirectTypes(instance));
    }

    public void testGetSlots() {
        Cls standardSlot = (Cls) getFrame(Model.Cls.STANDARD_SLOT);
        Cls slotMetaCls = createCls(standardSlot);
        Collection standardSlots = _testFrameStore.getSlots();
        Slot slot = createSlot();
        Slot slotb = createSlot(slot);
        Slot slotc = createSlotOnCls(slotMetaCls);
        Collection results = makeList(slot, slotb, slotc);
        results.addAll(standardSlots);
        assertEqualsSet("slots", results, _testFrameStore.getSlots());
    }
    public void testGetFacets() {
        Cls standardFacet = (Cls) getFrame(Model.Cls.STANDARD_FACET);
        Cls facetMetaCls = createCls(standardFacet);
        Collection standardFacets = _testFrameStore.getFacets();
        Facet facet = createFacet();
        Facet facetb = createFacet();
        Facet facetc = createFacet(ValueType.STRING, facetMetaCls);
        Collection results = makeList(facet, facetb, facetc);
        results.addAll(standardFacets);
        assertEqualsSet("slots", results, _testFrameStore.getFacets());
    }
    public void testGetMatchingReferences() {
        Cls cls = createCls();
        Slot slot = createSlotOnCls(cls, ValueType.STRING, true);
        Instance instance = createSimpleInstance(cls);
        int exactStartSize = _testFrameStore.getMatchingReferences("refxxx", FrameStore.UNLIMITED_MATCHES).size();
        int startsWithStartSize = _testFrameStore.getMatchingReferences("refxxx*", FrameStore.UNLIMITED_MATCHES).size();
        int containsStartSize = _testFrameStore.getMatchingReferences("*EFx*", FrameStore.UNLIMITED_MATCHES).size();
        _modifiableFrameStore.setDirectOwnSlotValues(instance, slot, makeList("refxxx", "erexxx"));
        Instance instance2 = createSimpleInstance(cls);
        _modifiableFrameStore.setDirectOwnSlotValues(instance2, slot, makeList("refxxxerexxx"));
        Collection references = _testFrameStore.getMatchingReferences("refxxx", FrameStore.UNLIMITED_MATCHES);
        assertEquals("exact", exactStartSize + 1, references.size());
        Collection references2 = _testFrameStore.getMatchingReferences("refxxx*", FrameStore.UNLIMITED_MATCHES);
        assertEquals("startswith", startsWithStartSize + 2, references2.size());
        Collection references3 = _testFrameStore.getMatchingReferences("*EFx*", FrameStore.UNLIMITED_MATCHES);
        assertEquals("contains", containsStartSize + 2, references3.size());
    }

    public void testGetFramesWithOwnSlotValue() {
        Cls cls = createCls();
        Slot slot = createSlotOnCls(cls, ValueType.STRING, true);
        Instance instancea = createSimpleInstance(cls);
        _modifiableFrameStore.setDirectOwnSlotValues(instancea, slot, makeList("own", "value"));
        Instance instanceb = createSimpleInstance(cls);
        _modifiableFrameStore.setDirectOwnSlotValues(instanceb, slot, makeList("foo", "own"));
        assertEqualsSet(
            "values",
            makeList(instancea, instanceb),
            _testFrameStore.getFramesWithDirectOwnSlotValue(slot, "own"));
    }

    public void testGetFramesWithMatchingOwnSlotValue() {
        Cls cls = createCls();
        Slot slot = createSlotOnCls(cls, ValueType.STRING, true);
        Instance instancea = createSimpleInstance(cls);
        _modifiableFrameStore.setDirectOwnSlotValues(instancea, slot, makeList("own", "value"));
        Instance instanceb = createSimpleInstance(cls);
        _modifiableFrameStore.setDirectOwnSlotValues(instanceb, slot, makeList("foo", "own"));
        Instance instancec = createSimpleInstance(cls);
        _modifiableFrameStore.setDirectOwnSlotValues(instancec, slot, makeList("owx"));
        assertEqualsSet(
            "exact",
            makeList(instancea, instanceb),
            _testFrameStore.getFramesWithMatchingDirectOwnSlotValue(slot, "own", FrameStore.UNLIMITED_MATCHES));
        assertEqualsSet(
            "starts with",
            makeList(instancea, instanceb, instancec),
            _testFrameStore.getFramesWithMatchingDirectOwnSlotValue(slot, "OW*", FrameStore.UNLIMITED_MATCHES));
    }

    public void testGetTemplateFacets() {
        Cls standardSlot = (Cls) getFrame(Model.Cls.STANDARD_SLOT);
        Slot associatedFacetSlot = (Slot) getFrame(Model.Slot.ASSOCIATED_FACET);
        Cls slotMetaCls = createCls(standardSlot);
        Slot metaClsSlot = createSlotOnCls(slotMetaCls);
        Facet facet = createFacet();
        Cls cls = createCls();
        Slot slot = createSlotOnCls(cls, slotMetaCls);
        setOwnSlotValue(metaClsSlot, associatedFacetSlot, facet);
        Collection facets = _testFrameStore.getTemplateFacets(cls, slot);
        assertTrue("new facet", facets.contains(facet));
    }

    public void testSetInverseSlotValues() {
        Cls cls = createCls();
        Slot slota = createSlotOnCls(cls, ValueType.INSTANCE, true);
        Slot slotb = createSlotOnCls(cls, ValueType.INSTANCE, true);
        Slot inverseSlotSlot = (Slot) getFrame(Model.Slot.INVERSE);
        setOwnSlotValue(slota, inverseSlotSlot, slotb);
        Instance instance1 = createSimpleInstance(cls);
        Instance instance2 = createSimpleInstance(cls);
        Cls clsc = createCls();
        Instance instance3 = createSimpleInstance(clsc);
        assertEquals(0, _testFrameStore.getDirectOwnSlotValues(instance1, slota).size());
        _testFrameStore.setDirectOwnSlotValues(instance1, slota, makeList(instance2));
        assertEqualsList("1", makeList(instance2), _testFrameStore.getDirectOwnSlotValues(instance1, slota));
        assertEqualsList("2", makeList(instance1), _testFrameStore.getDirectOwnSlotValues(instance2, slotb));
        _testFrameStore.setDirectOwnSlotValues(instance2, slotb, makeList(instance3));
        assertEquals(0, _testFrameStore.getDirectOwnSlotValues(instance1, slota).size());
        assertEqualsList("3", makeList(instance3), _testFrameStore.getDirectOwnSlotValues(instance2, slotb));
        assertEquals(0, _testFrameStore.getDirectOwnSlotValues(instance3, slota).size());
    }

    /*
    public void testGetEvents()  {
    }
    public void testExecuteQuery()  {
    }
    */
}