/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2001.  All Rights Reserved.
 *
 * Protege-2000 was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu
 *
 * Contributor(s):
 */

package edu.stanford.smi.protege.model.framestore;

import java.util.*;

import edu.stanford.smi.protege.model.*;
import edu.stanford.smi.protege.model.query.*;

/**
 * All queries go to all frame stores.  Writes go to the primary (delegate) frame store.
 * 
 * @author Ray Fergerson <fergerson@smi.stanford.edu>
 */
public class MergingBasicFrameStore implements BasicFrameStore {
    private List frameStores = new ArrayList();

    public void setDelegate(BasicFrameStore fs) {
        if (frameStores.isEmpty()) {
            frameStores.add(fs);
        } else {
            frameStores.set(0, fs);
        }
    }

    public BasicFrameStore getDelegate() {
        return (BasicFrameStore) frameStores.get(0);
    }

    public void addFrameStore(BasicFrameStore fs) {
        frameStores.add(fs);
    }

    public void removeFrameStore(BasicFrameStore fs) {
        frameStores.remove(fs);
    }

    public FrameID generateFrameID() {
        return getDelegate().generateFrameID();
    }

    public int getFrameCount() {
        int count = 0;
        Iterator i = frameStores.iterator();
        while (i.hasNext()) {
            BasicFrameStore fs = (BasicFrameStore) i.next();
            count += fs.getFrameCount();
        }
        return count;
    }

    public int getClsCount() {
        int count = 0;
        Iterator i = frameStores.iterator();
        while (i.hasNext()) {
            BasicFrameStore fs = (BasicFrameStore) i.next();
            count += fs.getClsCount();
        }
        return count;
    }

    public int getSlotCount() {
        int count = 0;
        Iterator i = frameStores.iterator();
        while (i.hasNext()) {
            BasicFrameStore fs = (BasicFrameStore) i.next();
            count += fs.getSlotCount();
        }
        return count;
    }

    public int getFacetCount() {
        int count = 0;
        Iterator i = frameStores.iterator();
        while (i.hasNext()) {
            BasicFrameStore fs = (BasicFrameStore) i.next();
            count += fs.getFacetCount();
        }
        return count;
    }

    public int getSimpleInstanceCount() {
        int count = 0;
        Iterator i = frameStores.iterator();
        while (i.hasNext()) {
            BasicFrameStore fs = (BasicFrameStore) i.next();
            count += fs.getSimpleInstanceCount();
        }
        return count;
    }

    public Frame getFrame(FrameID id) {
        Frame frame = null;
        Iterator i = frameStores.iterator();
        while (i.hasNext() && frame == null) {
            BasicFrameStore fs = (BasicFrameStore) i.next();
            frame = fs.getFrame(id);
        }
        return frame;
    }

    public List getValues(Frame frame, Slot slot, Facet facet, boolean isTemplate) {
        List values = new ArrayList();
        Iterator i = frameStores.iterator();
        while (i.hasNext() && frame == null) {
            BasicFrameStore fs = (BasicFrameStore) i.next();
            values.addAll(fs.getValues(frame, slot, facet, isTemplate));
        }
        return values;
    }

    public int getValuesCount(Frame frame, Slot slot, Facet facet, boolean isTemplate) {
        int count = 0;
        Iterator i = frameStores.iterator();
        while (i.hasNext()) {
            BasicFrameStore fs = (BasicFrameStore) i.next();
            count += fs.getValuesCount(frame, slot, facet, isTemplate);
        }
        return count;
    }

    public void addValues(Frame frame, Slot slot, Facet facet, boolean isTemplate, Collection values) {
        getDelegate().addValues(frame, slot, facet, isTemplate, values);
    }

    public void moveValue(Frame frame, Slot slot, Facet facet, boolean isTemplate, int from, int to) {
        getDelegate().moveValue(frame, slot, facet, isTemplate, from, to);
    }

    public void removeValue(Frame frame, Slot slot, Facet facet, boolean isTemplate, Object value) {
        getDelegate().removeValue(frame, slot, facet, isTemplate, value);
    }

    public void setValues(Frame frame, Slot slot, Facet facet, boolean isTemplate, Collection values) {
        getDelegate().setValues(frame, slot, facet, isTemplate, values);
    }

    public Set getFrames(Slot slot, Facet facet, boolean isTemplate, Object value) {
        Set frames = new HashSet();
        Iterator i = frameStores.iterator();
        while (i.hasNext()) {
            BasicFrameStore fs = (BasicFrameStore) i.next();
            frames.addAll(fs.getFrames(slot, facet, isTemplate, value));
        }
        return frames;
    }

    public Set getFramesWithAnyValue(Slot slot, Facet facet, boolean isTemplate) {
        Set frames = new HashSet();
        Iterator i = frameStores.iterator();
        while (i.hasNext()) {
            BasicFrameStore fs = (BasicFrameStore) i.next();
            frames.addAll(fs.getFramesWithAnyValue(slot, facet, isTemplate));
        }
        return frames;
    }
    public Set getMatchingFrames(Slot slot, Facet facet, boolean isTemplate, String value, int maxMatches) {
        Set frames = new HashSet();
        Iterator i = frameStores.iterator();
        while (i.hasNext() && frames.size() < maxMatches) {
            BasicFrameStore fs = (BasicFrameStore) i.next();
            frames.addAll(fs.getMatchingFrames(slot, facet, isTemplate, value, maxMatches - frames.size()));
        }
        return frames;
    }
    public Set getReferences(Object value) {
        Set references = new HashSet();
        Iterator i = frameStores.iterator();
        while (i.hasNext()) {
            BasicFrameStore fs = (BasicFrameStore) i.next();
            references.addAll(fs.getReferences(value));
        }
        return references;
    }
    public Set getMatchingReferences(String value, int maxMatches) {
        Set references = new HashSet();
        Iterator i = frameStores.iterator();
        while (i.hasNext() && references.size() < maxMatches) {
            BasicFrameStore fs = (BasicFrameStore) i.next();
            references.addAll(fs.getMatchingReferences(value, maxMatches - references.size()));
        }
        return references;
    }

    public Set executeQuery(Query query) {
        Set results = new HashSet();
        Iterator i = frameStores.iterator();
        while (i.hasNext()) {
            BasicFrameStore fs = (BasicFrameStore) i.next();
            results.addAll(fs.executeQuery(query));
        }
        return results;
    }

    public Set getClosure(Frame frame, Slot slot, Facet facet, boolean isTemplate) {
        Set frames = new HashSet();
        Iterator i = frameStores.iterator();
        while (i.hasNext()) {
            BasicFrameStore fs = (BasicFrameStore) i.next();
            frames.addAll(fs.getClosure(frame, slot, facet, isTemplate));
        }
        return frames;
    }

    public void deleteFrame(Frame frame) {
        getDelegate().deleteFrame(frame);
    }
    public void close() {
        Iterator i = frameStores.iterator();
        while (i.hasNext()) {
            BasicFrameStore fs = (BasicFrameStore) i.next();
            fs.close();
        }
        frameStores.clear();
    }

    public void replaceFrame(Frame frame) {
        getDelegate().replaceFrame(frame);
    }
    public boolean beginTransaction(String name) {
        return getDelegate().beginTransaction(name);
    }
    public boolean commitTransaction() {
        return getDelegate().commitTransaction();
    }
    public boolean rollbackTransaction() {
        return getDelegate().rollbackTransaction();
    }
}