/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2001.  All Rights Reserved.
 *
 * Protege-2000 was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu
 *
 * Contributor(s):
 */

package edu.stanford.smi.protege.ui;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.util.List;

import javax.swing.*;

import edu.stanford.smi.protege.action.*;
import edu.stanford.smi.protege.event.*;
import edu.stanford.smi.protege.model.*;
import edu.stanford.smi.protege.util.*;

/**
 * The panel that holds the list of direct instances of one or more classes.  If only one class is chosen then you 
 * can also create new instances of this class.
 *
 * @author    Ray Fergerson <fergerson@smi.stanford.edu>
 */
public class DirectInstancesList extends SelectableContainer implements Disposable {
    private Collection _clses = Collections.EMPTY_LIST;
    private SelectableList _list;
    private SetDisplaySlotPanel _setDisplaySlotPanel;
    private JComboBox _directInstancesComboBox;
    private Project _project;
    private AllowableAction _createAction;
    private AllowableAction _copyAction;
    private AllowableAction _deleteAction;
    private Collection listenedToInstances = new ArrayList();
    private static final int SORT_LIMIT;
    public static final String DIRECT = "Direct Instances";
    public static final String ALL = "All Instances";

    static {
        SORT_LIMIT = ApplicationProperties.getIntegerProperty("ui.DirectInstancesList.sort_limit", 1000);
    }

    private ClsListener _clsListener = new ClsAdapter() {
        public void directInstanceAdded(ClsEvent event) {
            Instance instance = event.getInstance();
            ComponentUtilities.addListValue(_list, instance);
            instance.addFrameListener(_instanceFrameListener);
        }

        public void directInstanceRemoved(ClsEvent event) {
            removeInstance(event.getInstance());
        }
    };
    private FrameListener _clsFrameListener = new FrameAdapter() {
        public void ownSlotValueChanged(FrameEvent event) {
            super.ownSlotValueChanged(event);
            updateCreateButton((Cls) event.getFrame());
        }
    };
    private FrameListener _instanceFrameListener = new FrameAdapter() {
        public void browserTextChanged(FrameEvent event) {
            super.browserTextChanged(event);
            // Log.enter(this, "browserTextChanged", event);
            sort();
            repaint();
        }
    };

    public DirectInstancesList(Project project) {
        _project = project;
        Action viewAction = createViewAction();

        _list = ComponentFactory.createSelectableList(viewAction);
        _list.setCellRenderer(FrameRenderer.createInstance());
        _list.setPrototypeCellValue(_project.getKnowledgeBase().getRootCls());
        _setDisplaySlotPanel = new SetDisplaySlotPanel();
        setLayout(new BorderLayout(10, 10));
        LabeledComponent c = new LabeledComponent(null, new JScrollPane(_list));
        c.setHeaderComponent(createDirectAllComboBox(), BorderLayout.WEST);
        addButtons(viewAction, c);
        c.setFooterComponent(new ListFinder(_list, "Find Instance"));
        add(c, BorderLayout.CENTER);
        add(_setDisplaySlotPanel, BorderLayout.NORTH);

        setSelectable(_list);
        setShowDisplaySlotPanel(true);
    }

    private JComboBox createDirectAllComboBox() {
        _directInstancesComboBox = ComponentFactory.createComboBox();
        _directInstancesComboBox.addItem(DIRECT);
        _directInstancesComboBox.addItem(ALL);
        _directInstancesComboBox.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent event) {
                reload();
                fixRenderer();
            }
        });
        return _directInstancesComboBox;
    }

    public void setShowDisplaySlotPanel(boolean b) {
        _setDisplaySlotPanel.setVisible(b);
        revalidate();
    }

    private void fixRenderer() {
        boolean displayType = _directInstancesComboBox.getSelectedItem().equals(ALL);
        FrameRenderer frameRenderer = (FrameRenderer) _list.getCellRenderer();
        frameRenderer.setDisplayType(displayType);
    }

    protected void addButtons(Action viewAction, LabeledComponent c) {
        c.addHeaderButton(viewAction);
        c.addHeaderButton(createCreateAction());
        c.addHeaderButton(createCopyAction());
        c.addHeaderButton(createReferencersAction());
        c.addHeaderButton(createDeleteAction());
    }

    private void addClsListeners() {
        Iterator i = _clses.iterator();
        while (i.hasNext()) {
            Cls cls = (Cls) i.next();
            cls.addClsListener(_clsListener);
            cls.addFrameListener(_clsFrameListener);
        }
    }

    private void addInstanceListeners() {
        ListModel model = _list.getModel();
        int start = _list.getFirstVisibleIndex();
        int stop = _list.getLastVisibleIndex();
        for (int i = start; i < stop; ++i) {
            Instance instance = (Instance) model.getElementAt(i);
            addInstanceListener(instance);

        }
    }

    private void removeInstanceListeners() {
        Iterator i = listenedToInstances.iterator();
        while (i.hasNext()) {
            Instance instance = (Instance) i.next();
            instance.removeFrameListener(_instanceFrameListener);
        }
        listenedToInstances.clear();
    }

    private void addInstanceListener(Instance instance) {
        instance.addFrameListener(_instanceFrameListener);
        listenedToInstances.add(instance);
    }

    protected Action createCreateAction() {
        _createAction = new CreateAction("Create Instance") {
            public void onCreate() {
                Cls cls = (Cls) CollectionUtilities.getFirstItem(_clses);
                if (cls != null) {
                    KnowledgeBase kb = _project.getKnowledgeBase();
                    Instance instance = kb.createInstance(null, cls);
                    if (instance instanceof Cls) {
                        Cls newCls = (Cls) instance;
                        if (newCls.getDirectSuperclassCount() == 0) {
                            newCls.addDirectSuperclass(kb.getRootCls());
                        }
                    }
                    _list.setSelectedValue(instance, true);
                }
            }
        };
        return _createAction;
    }

    protected Action createDeleteAction() {
        _deleteAction = new DeleteInstancesAction("Delete Selected Instances", this);
        return _deleteAction;
    }

    protected Action createCopyAction() {
        _copyAction = new CopyAction("Copy Selected Instances", this) {
            protected Instance copy(Instance instance, boolean isDeep) {
                Instance copy = super.copy(instance, isDeep);
                setSelectedInstance(copy);
                return copy;
            }
        };
        return _copyAction;
    }

    protected Action createReferencersAction() {
        return new ReferencersAction(this);
    }

    protected Action createViewAction() {
        return new ViewAction("View Instance", this) {
            public void onView(Object o) {
                _project.show((Instance) o);
            }
        };
    }

    public void dispose() {
        removeClsListeners();
        removeInstanceListeners();
    }

    public JComponent getDragComponent() {
        return _list;
    }

    private SimpleListModel getModel() {
        return (SimpleListModel) _list.getModel();
    }

    private boolean isSelectionEditable() {
        boolean isEditable = true;
        Iterator i = getSelection().iterator();
        while (i.hasNext()) {
            Instance instance = (Instance) i.next();
            if (!instance.isEditable()) {
                isEditable = false;
                break;
            }
        }
        return isEditable;
    }

    public void onSelectionChange() {
        // Log.enter(this, "onSelectionChange");
        boolean editable = isSelectionEditable();
        ComponentUtilities.setDragAndDropEnabled(_list, editable);
    }

    private void removeInstance(Instance instance) {
        ComponentUtilities.removeListValue(_list, instance);
        instance.removeFrameListener(_instanceFrameListener);
    }

    private void removeClsListeners() {
        Iterator i = _clses.iterator();
        while (i.hasNext()) {
            Cls cls = (Cls) i.next();
            cls.removeClsListener(_clsListener);
            cls.removeFrameListener(_clsFrameListener);
        }
    }

    public void setClses(Collection newClses) {
        removeClsListeners();
        _clses = new ArrayList(newClses);
        reload();
        Cls selectedCls = (_clses.size() == 1) ? (Cls) CollectionUtilities.getFirstItem(_clses) : null;
        updateCreateButton(selectedCls);
        _setDisplaySlotPanel.setCls(selectedCls);

        addClsListeners();
    }

    public void reload() {
        removeInstanceListeners();
        Object selectedValue = _list.getSelectedValue();
        ArrayList instances = new ArrayList();
        Iterator i = _clses.iterator();
        while (i.hasNext()) {
            Cls cls = (Cls) i.next();
            instances.addAll(getInstances(cls));
        }
        if (instances.size() <= SORT_LIMIT) {
            Collections.sort(instances, new FrameComparator());
        }
        getModel().setValues(instances);
        if (instances.contains(selectedValue)) {
            _list.setSelectedValue(selectedValue, true);
        } else if (!instances.isEmpty()) {
            _list.setSelectedIndex(0);
        }
        addInstanceListeners();
    }

    private Collection getInstances(Cls cls) {
        Collection instances;
        if (_directInstancesComboBox.getSelectedItem().equals(DIRECT)) {
            instances = cls.getDirectInstances();
        } else {
            instances = cls.getInstances();
        }
        if (!_project.getDisplayHiddenFrames()) {
            instances = removeHiddenInstances(instances);
        }
        return instances;
    }

    private Collection removeHiddenInstances(Collection instances) {
        Collection visibleInstances = new ArrayList(instances);
        Iterator i = visibleInstances.iterator();
        while (i.hasNext()) {
            Instance instance = (Instance) i.next();
            if (!instance.isVisible()) {
                i.remove();
            }
        }
        return visibleInstances;
    }

    public void sort() {
        _list.setListenerNotificationEnabled(false);
        Object selectedValue = _list.getSelectedValue();
        List instances = new ArrayList(getModel().getValues());
        if (instances.size() <= SORT_LIMIT) {
            Collections.sort(instances, new FrameComparator());
        }
        getModel().setValues(instances);
        _list.setSelectedValue(selectedValue);
        _list.setListenerNotificationEnabled(true);
    }

    public void setSelectedInstance(Instance instance) {
        _list.setSelectedValue(instance, true);
    }

    private void updateCreateButton(Cls cls) {
        _createAction.setEnabled(cls == null ? false : cls.isConcrete());
    }
}
